package infomat.evaluation;

import java.util.HashMap;
import java.util.Vector;
import java.io.*;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.measures.*;
import mro.util.properties.*;


/**
 * For calculating quality measures.
 * 
 * @version          080608
 * @author           Magnus Rosell
 */
public class Evaluation implements HasProperties{

    private Properties properties;

    private IMatrix matrix;
    private IObjectGrouping evalGrouping = null;
    private IObjectGrouping refGrouping = null;
    private ISimilarity similarity = null;
    private String stamp;

    private static String evalGroupingDesignation = "Evaluated Grouping";
    private static String refGroupingDesignation = "Reference Grouping";


    public Evaluation(IMatrix matrix, 
		      IObjectGrouping evalGrouping, 
		      IObjectGrouping refGrouping, 
		      ISimilarity similarity, 
		      String stamp){
	this.matrix = matrix;
	this.evalGrouping = evalGrouping;
	this.refGrouping = refGrouping;
	this.similarity = similarity;
	this.stamp=stamp;
	this.properties = getDefaultProperties();
    }

    public Evaluation(IMatrix matrix, 
		      IObjectGrouping evalGrouping, 
		      ISimilarity similarity, 
		      String stamp){
	this.matrix = matrix;
	this.evalGrouping = evalGrouping;
	this.similarity = similarity;
	this.stamp=stamp;
	this.properties = getDefaultProperties();
    }


    public Properties getDefaultProperties(){

	Properties properties = 
	    new Properties("Evaluation Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    Property p = new Property("Extrinsic Evaluation", true, "Extrinsic Evaluation");
	    PropertyValueDescription pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Intrinsic Evaluation", true, "Intrinsic Evaluation");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Group Evaluation", true, "Evaluate groups individually");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }
    



    public Measures calculateAllMeasures() 
	throws MeasureNameConflictException{
	System.out.println("calculateAllMeasures()");

	Measures measures = new Measures("IObjectGrouping Evaluation");

	String groupEvaluationString = "Group Evaluation";
	boolean extrinsicEvaluation = (Boolean) 
	    getPropertyValue("Extrinsic Evaluation");
	boolean intrinsicEvaluation = (Boolean) 
	    getPropertyValue("Intrinsic Evaluation");

	if(refGrouping!=null && extrinsicEvaluation){

	    System.out.println("extrinsicEvaluation");

	    ExtrinsicEvaluation em = new ExtrinsicEvaluation(evalGrouping, refGrouping, stamp);
	    try{
		em.setProperty(groupEvaluationString, getPropertyValue(groupEvaluationString));
	    }catch(PropertyNotFoundException pnfe){
		// Should not happen here
	    }
	    MeasureGroup mg = em.allMeasuresInAGroup();
	    measures.addMeasure(mg);

	    System.out.println("extrinsicEvaluation");
	}

	MeasureGroup intrinsicGroup = new MeasureGroup("Intrinsic Measures");

	if(intrinsicEvaluation){
	    System.out.println("intrinsicEvaluation");

	    System.out.println("intrinsicEvaluation - eval grouping");
	    IntrinsicEvaluation ieEval = 
		new IntrinsicEvaluation(evalGrouping, similarity, evalGroupingDesignation, stamp);
	    System.out.println("intrinsicEvaluation - eval grouping 2");
	    try{
		ieEval.setProperty(groupEvaluationString, getPropertyValue(groupEvaluationString));
		System.out.println("intrinsicEvaluation - eval grouping 3");
	    
	    }catch(PropertyNotFoundException pnfe){
		// Should not happen here
		System.out.println("intrinsicEvaluation - PropertyNotFoundException");
	    }
	    MeasureGroup mg = ieEval.allGlobalMeasuresInAGroup();
	    System.out.println("intrinsicEvaluation - eval grouping 4");
	    intrinsicGroup.addMeasureVector(mg.getMeasureVector());
	    System.out.println("intrinsicEvaluation - eval grouping");

	    if(refGrouping!=null){
		System.out.println("intrinsicEvaluation - ref grouping");
		IntrinsicEvaluation ieRef = 
		    new IntrinsicEvaluation(refGrouping, similarity, refGroupingDesignation, stamp);
		try{
		    ieRef.setProperty(groupEvaluationString, getPropertyValue(groupEvaluationString));
		}catch(PropertyNotFoundException pnfe){
		    // Should not happen here
		}
		mg = ieRef.allGlobalMeasuresInAGroup();
		intrinsicGroup.addMeasureVector(mg.getMeasureVector());
		System.out.println("intrinsicEvaluation - ref grouping");
	    }
	    
	    //	    intrinsicGroup.addMeasure(ieEval.calculateBasicMatrixMeasures());

	    boolean groupEvaluation = (Boolean) 
		getPropertyValue(groupEvaluationString);
	    if(groupEvaluation)
		intrinsicGroup.addMeasure(ieEval.calculateGroupMeasures());

	    measures.addMeasure(intrinsicGroup);
	    System.out.println("intrinsicEvaluation");
	}

	if(matrix!=null){
	    System.out.println("MatrixMeasures");
	    MatrixMeasures mm = 
		new MatrixMeasures(matrix, 
				   evalGrouping, 
				   refGrouping, 
				   evalGroupingDesignation, 
				   refGroupingDesignation, 
				   stamp);
	    measures.addMeasure(mm.allMeasuresInAGroup());
	    System.out.println("MatrixMeasures");
	}

	measures.setAllMeasuresStampsRecursively(stamp);

	return measures;
    }


    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public Properties getProperties(){
    	return properties;
    }

    public void setProperties(Properties properties){
    	this.properties=properties;
    }
}
