package infomat.vectorspace;

import mro.util.CommonPath;

import java.util.*;

/**
 * A IObjectSet is a complete set of IObject:s.
 * This can either be the complete set of objects one investigates
 * or the complete feature set by which these objects are represented.
 * In fact, this representations makes no difference between the two.
 *
 * @param groupings A vector of groupings of this IObjectSet.
 * @param nameGroupingMap A mapping of the names of the IObjectGroupings of this IObjectSet to the IObjectGroupings.
 * @param currentGrouping The current IObjectGrouping. This is the basis for the ordering of the IObject:s in theis IOBjectSet right now.
 * @param idIObjectMap A mapping between IObjectId and IObject.
 *
 * @version          070314
 * @author           Magnus Rosell
 */
public class IObjectSet extends IObjectGroup
{
    private static int idIterator=0;

    private Vector<IObjectGrouping> groupings;
    private HashMap<String, IObjectGrouping> nameGroupingMap;
    private IObjectGrouping currentGrouping=null;
    private HashMap<Integer, IObject> idIObjectMap;
    private HashMap<String, HashSet<IObject>> stringAmbiguousIObjectsMap;

    public IObjectSet(String name){ 
	super(name, 3, idIterator++); // 3 - IObjectSet

	groupings = new Vector<IObjectGrouping>();
	nameGroupingMap = new HashMap<String, IObjectGrouping>();
	idIObjectMap = new HashMap<Integer, IObject>();
	stringAmbiguousIObjectsMap = new HashMap<String, HashSet<IObject>>();
    }


    /**
     * Copy constructor. Makes new instances of everything, 
     * but the IObject:s, which are the same.
     * Gives the new IObjectSet the same name, 
     * but a different id.
     */
    public IObjectSet(IObjectSet ios){
	super(ios, 3, idIterator++); 

	groupings = new Vector<IObjectGrouping>();
	nameGroupingMap = new HashMap<String, IObjectGrouping>();
	idIObjectMap = new HashMap<Integer, IObject>();
	//	stringNumberOfIObjects = new HashMap<String, Integer>();
	stringAmbiguousIObjectsMap = new HashMap<String, HashSet<IObject>>();

	Vector<IObjectGrouping> ings = ios.groupings;
	for(int i=0; i<ings.size(); i++){
	    addGrouping(new IObjectGrouping(ings.elementAt(i)));
	}
	currentGrouping = nameGroupingMap.get(ios.currentGrouping.getName());

	Vector<IObject> objs = getOrdered();
	for(int i=0; i<objs.size(); i++){
	    IObject io = objs.elementAt(i);
	    add(io);
	    //	    idIObjectMap.put(io.getID(), io);
	}
    }


    /**
     * Adds a new IObject.
     * OBS. This is not reflected in any IObjectGrouping:s 
     * that already exit. 
     */
    public void add(IObject io){
	super.add(io);
	idIObjectMap.put(io.getID(), io);


	// Ambiguous String?
	io.setString(io.getAmbiguousString());
	HashSet<IObject> set = 
	    stringAmbiguousIObjectsMap.get(io.getAmbiguousString());
	if(set==null){
	    set = new HashSet<IObject>();
	    set.add(io);
	    stringAmbiguousIObjectsMap.put(io.getAmbiguousString(), set);
	    io.setAmbiguous(false);
	}else{
	    if(set.size()==1){
		Iterator<IObject> it = set.iterator();
		while(it.hasNext())
		    it.next().setAmbiguous(true);
	    }
	    set.add(io);
	    io.setAmbiguous(true);
	}
	
    }

    public void add(IObject io, float orderValue){
	super.add(io, orderValue);
	idIObjectMap.put(io.getID(), io);

	// Ambiguous String?
	io.setString(io.getAmbiguousString());
	HashSet<IObject> set = 
	    stringAmbiguousIObjectsMap.get(io.getAmbiguousString());
	if(set==null){
	    set = new HashSet<IObject>();
	    set.add(io);
	    stringAmbiguousIObjectsMap.put(io.getAmbiguousString(), set);
	    io.setAmbiguous(false);
	}else{
	    if(set.size()==1){
		Iterator<IObject> it = set.iterator();
		while(it.hasNext())
		    it.next().setAmbiguous(true);
	    }
	    set.add(io);
	    io.setAmbiguous(true);
	}

    }


    /** 
     * Removes a IObject. It is removed from all IObjectGrouping:s as well.
     * Compare to #add where IObjects are not added to existing 
     * IObjectGrouping:s.
     */
    public void remove(IObject io){
	super.remove(io);
	idIObjectMap.remove(io.getID());

	if(groupings!=null){
	    for(int i=groupings.size()-1; i>=0; i--){
		IObjectGrouping ioging = groupings.elementAt(i);
		ioging.remove(io);
		if(ioging.numberOfIObjectGroups()<1) // Hnder detta r det fara  frde! :)
		    removeGrouping(ioging);
	    }
	}



	// Ambiguous String?
	HashSet<IObject> set = 
	    stringAmbiguousIObjectsMap.get(io.getAmbiguousString());
	if(set!=null){
	    if(set.size()==1){
		stringAmbiguousIObjectsMap.remove(io.getAmbiguousString());
	    }else{
		set.remove(io);
		if(set.size()==1){
		    Iterator<IObject> it = set.iterator();
		    while(it.hasNext())
			it.next().setAmbiguous(false);
		}
	    }
	}

	/*
	  SKa inte behvas - alla grupperingar ska finnas i vektorn.
	  if(currentGrouping!=null)
	  currentGrouping.remove(io);
	*/
    }

    public void removeAll(){
	super.removeAll();
	groupings = new Vector<IObjectGrouping>();
	nameGroupingMap = new HashMap<String, IObjectGrouping>();
	idIObjectMap = new HashMap<Integer, IObject>();
	stringAmbiguousIObjectsMap = new HashMap<String, HashSet<IObject>>();
    }

    public void sortInCurrentVectorOrderWithInvertedOrderNumberAsValue(){
	Vector<IObject> vec = getOrdered();
	removeAll();
	for(int i=0; i<vec.size(); i++){
	    add(vec.get(i), vec.size()-i);
	}
    }



    /**
     * Removes and returns the IObjects in the range given,
     * the range within the vector "ordered".
     */
    /*
      protected Vector<IObject> remove(int from, int to){

      // Ta bort
      Vector<IObject> removed = new Vector<IObject>();
      for (int i=to;i>=from;i--){
      IObject co = ordered.elementAt(i);
      remove(co);
      removed.add(co);
      }

      // Sortera om
      if(groupings!=null){
      for(int i=0; i<groupings.size(); i++){
      IObjectGrouping coging = groupings.elementAt(i);
      coging.sortAllGroupsInOrderValueOrder();
      }
      }
      return removed;
      }
    */

    /**
     * Adds an IObjectGrouping.
     * Returns false if the name for this new grouping is already 
     * taken. 
     */
    public boolean addGrouping(IObjectGrouping cog){
	String name = cog.getName();
	if(nameGroupingMap.containsKey(name))
	    return false;
	groupings.add(cog);
	nameGroupingMap.put(name, cog);
	return true;
    }

    /**
     * Removes an IObjectGrouping.
     */
    public void removeGrouping(IObjectGrouping cog){
	groupings.remove(cog);
	nameGroupingMap.remove(cog.getName());
    }

    /**
     * Removes all IObjects that's not in the current IObjectGrouping
     * from the set and all its IObjectGroupings.
     */
    public void purgeSetToCurrentGrouping(){
	Vector<IObject> removed = new Vector<IObject>();
	for(int i=0; i<ordered.size(); i++){
	    IObject io = ordered.elementAt(i);
	    if(currentGrouping.getIObjectOrderIndex(io)==-1)
		removed.add(io);
	}
	for(int i=0; i<removed.size(); i++){
	    IObject io = removed.elementAt(i);
	    // remove from set
	    remove(io); 
	    // remove from groupings
	    for(int j=0; j<groupings.size(); j++){
		groupings.elementAt(j).remove(io);
	    }
	}
	sortInOrderValueOrder();
    }

    /**
     * Removed IObject:s from the IObjectGrouping:s that are not in the
     * IObjectSet. Used by IMatrixCellFilter.
     *
     * @see IMatrixFilter, IObjectGrouping#prugeGroupingToSet()
     */
    protected void purgeGroupingsToSet(){
	for(int j=0; j<groupings.size(); j++){
	    groupings.elementAt(j).purgeGroupingToSet(this);
	}	
    }

    /**
     * Sets the current IObjectGrouping.
     * Returns false if the IObjectGrouping has a conflict
     * as in #addGrouping(). 
     */
    public boolean setGrouping(IObjectGrouping cog){
	boolean ret=true;
	if(cog!=null && !groupings.contains(cog))
	    ret = addGrouping(cog);
	if(ret)
	    currentGrouping=cog;
	return ret;
    }

    /**
     * Returns the current IObjectGrouping.
     */
    public IObjectGrouping getCurrentGrouping(){
	return currentGrouping;
    }

    /**
     * Returns all IObjectGrouping:s of this IObjectSet.
     */
    public Vector<IObjectGrouping> getGroupings(){
	return groupings;
    }

    /**
     * Returns the IObjectGrouping with the name given.
     */
    public IObjectGrouping getGroupingWithName(String name){
	return nameGroupingMap.get(name);
    }

    /**
     * Returns true if the IObjectGrouping orders this IObjectSet.
     */
    public boolean containsIObjectGrouping(IObjectGrouping ioging){
	IObjectGrouping dummy = nameGroupingMap.get(ioging.getName());
	if(dummy!=null)
	    return true;
	return false;
    }

    /**
     * Returns true if any of the IObjectGrouping:s that order this
     * IObjectSet contains the IObjectGroup.
     */
    public boolean containsIObjectGroup(IObjectGroup iog){
	for(int i=0; i<groupings.size(); i++){
	    IObjectGrouping ioging = groupings.elementAt(i);
	    if(ioging.containsIObjectGroup(iog))
		return true;
	}
	return false;
    }

    /**
     * Sorts the vector "ordered" accorging to the IObjectGrouping given.
     */
    /*
      public void sortAccordingToGrouping(IObjectGrouping coging){
      ordered = new Vector<IObject>();
      objectOrderIndexMap = new HashMap<IObject, Integer>();
      objectOrderValueMap = new HashMap<IObject, Float>();
      idIObjectMap = new HashMap<Integer, IObject>();
      Vector<IObjectGroup> groups = coging.getGroups();
      for(int i=0; i<groups.size(); i++){
      Vector<IObject> cobjects = groups.elementAt(i).getOrdered();
      for(int j=0; j<cobjects.size(); j++){
      IObject co = cobjects.elementAt(j);
      add(co);
      }
      }

      }
    */

    /**
     * Makes a IObjectGrouping consisting of one IObjectGroup and
     * sets it to be the current IObjectGrouping. Used as a first 
     * IObjectGrouping as many function needs one to work.
     */
    public IObjectGrouping makeStartGrouping(){ 
	IObjectGrouping startGrouping = 
	    new IObjectGrouping("Initial Grouping for "+getString());
	int id=-1;
	IObjectGroup startGroup = 
	    new IObjectGroup("Initial Grouping Group", id); //, myIMatrix);
	startGrouping.add(startGroup);

	Vector<IObject> ordered = getOrdered();
	for(int i=ordered.size()-1; i>=0; i--){
	    IObject co = ordered.elementAt(i);
	    startGroup.add(co);
	}

	startGrouping.sort();
	int i=1;
	while(!setGrouping(startGrouping)){
	    startGrouping.setName(startGrouping.getName()+" ("+i+")");
	    i++;
	}
	return startGrouping;
    }

    /**
     * Returns the vector "ordered" in its current order.
     * If it isn't assign it is ordered accoring to the current
     * IObjectGrouping first.
     */
    /*
      public Vector<IObject> getOrdered(){
      if(currentGrouping==null)
      return ordered;
      else{
      sortAccordingToGrouping(currentGrouping);
      return ordered;
      }
      }
    */

    /**
     * Returns the IObject with the given id.
     */
    public IObject getIObjectWithId(int i){
	return idIObjectMap.get(i);
    }

    public String getLargestStartOfLocation(){
	String largest = null;
	Vector<IObject> vec = getOrdered();
	if(vec.size()<1 || vec.get(0).getLocation()==null)
	    return null;
	for(int i=0; i<vec.size(); i++){
	    IObject io = vec.get(i);
	    String next = io.getLocation();
	    if(largest==null)
		largest=next;
	    largest = CommonPath.commonStart(largest, next);
	}
	if(largest!=null && largest.length()==0)
	    return null;
	return largest;
    }
}

