package infomat.view.picture;

import java.util.*;
import java.awt.Color;

import infomat.vectorspace.*;
import infomat.view.grouping.settings.GroupingColoring;

/**
 * A SparsePixel is a non zero pixel in a SparsePartPicture.
 * It represents one or several IMatrixCells. Its opacity
 * dependens on the values of these and its color depends on
 * which IObjectGroups they belong to, in the current IObjectGrouping.
 */
public class SparsePixel
{
    private int x;
    private int y;
    private int xLast=-1;
    private int yLast=-1;

    // The color and opacity
    private float[] rgb;
    private float opacity=0;

    // The total weight of the matrix cells.
    private float weight=0;
	
    //    int numOfCells;
    private TreeSet<IMatrixCell> matrixCells;
    //    private IMatrixCell firstColumnCell;
    //    private IMatrixCell firstRowCell;

    //private boolean averaged=false;

    private static double numberOfIMatrixColumns=1;
    private static double numberOfIMatrixRows=1;


    public SparsePixel(int x, int y, Color color, IMatrixCell mc) {
	this.x=x;
	this.y=y;
	if(color==null){
	    rgb=null;
	    if(mc==null)
		this.weight=0f;
	    else
		this.weight = mc.getWeight();
	}
	else{
	    rgb = color.getRGBColorComponents(null);
	    this.weight = mc.getWeight();
	}
	//	numOfCells=1;
	matrixCells = 
	    new TreeSet<IMatrixCell>(new IMatrixCellWeightComparator());
	if(mc!=null)
	    matrixCells.add(mc);
    }

    public SparsePixel(int x, int y, int xLast, int yLast,
		       Color color, IMatrixCell mc) {
	this.x=x;
	this.y=y;
	this.xLast=xLast;
	this.yLast=yLast;

	if(color==null){
	    rgb=null;
	    if(mc==null)
		this.weight=0f;
	    else
		this.weight = mc.getWeight();
	}
	else{
	    rgb = color.getRGBColorComponents(null);
	    this.weight = mc.getWeight();
	}
	//	numOfCells=1;
	matrixCells = 
	    new TreeSet<IMatrixCell>(new IMatrixCellWeightComparator());
	if(mc!=null)
	    matrixCells.add(mc);
    }


    /** 
     * Not a genuine copy-constructor! Copies references...
     */
    /*
      public SparsePixel(SparsePixel sp){
      x=sp.x;
      y=sp.y;
      rgb=sp.rgb;
      opacity=sp.opacity;
      //	numOfCells=sp.numOfCells;
      matrixCells=sp.matrixCells;
      }
    */
	
	
    public static void setNumberOfIMatrixColumns(double num){
	numberOfIMatrixColumns=num;
    }

    public static void setNumberOfIMatrixRows(double num){
	numberOfIMatrixRows=num;
    }

    /*
      public void accumulateColorPixel(SparsePixel sp){
      this.opacity+=sp.getOpacity();
      if(rgb!=null){
      float[] rgba=sp.getRGB();
      if(rgba!=null){
      for (int i=0;i<3;i++) {
      rgb[i] += rgba[i];
      }
      }
      }else{
      rgb=sp.getRGB();
      }
      }
    */
	
    public void accumulate(SparsePixel sp){
	this.weight+=sp.getWeight();

	matrixCells.addAll(sp.matrixCells);

	if(rgb!=null){
	    float[] rgba=sp.getRGB();
	    if(rgba!=null)
		rgb = addRGBs(rgb, rgba);
	}else{
	    rgb=sp.getRGB();
	}
    }


    public void recolor(boolean row, GroupingColoring coloring){
	rgb = new float[3];
		
	Vector<IMatrixCell> cells = new Vector<IMatrixCell>(matrixCells);
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = cells.elementAt(i);
	    IObject io = mc.getColumnIObject();
	    if(row)
		io = mc.getRowIObject();
	    Color color = coloring.getColorForObject(io);
	    float[] mcRGB = color.getRGBComponents(null);
	    rgb = addRGBs(rgb, mcRGB);
	}
	makeAverageColor();
    }
	
    private float[] addRGBs(float[] a, float[] b){
	float[] ret = new float[3];
	for(int i=0; i<ret.length; i++)
	    ret[i]=a[i]+b[i];
	return ret;
    }
	

    public void makeAverageColor(){
	int numOfCells = getNumberOfIMatrixCells();
	divideValuesBy(numOfCells);
    }

    private void divideValuesBy(int div){
	if(rgb!=null){
	    for (int i=0;i<3;i++) {
		rgb[i] = rgb[i]/div;
		if(rgb[i]>1)
		    rgb[i]=1;
	    }
	    /*
	      if(rgb[0]>1 || rgb[1]>1 || rgb[2]>1)
	      printColorParamters();
	    */
	}
    }

    /*
      public void linearScaleRGBComponents(float newMin, 
      float newMax, 
      float currentMin, 
      float currentMax){
      if(rgb!=null){
      for(int i=0; i<3; i++){
      if(rgb[i]>0)
      rgb[i] = (float)
      Math.min(1.0, rgb[i]/(currentMax-currentMin)*(newMax-newMin)+newMin);
      }
      }
      }
    */
	
    public void linearScaleOpacity(float newMin, float newMax, 
				   float currentMin, float currentMax){
	if((currentMax-currentMin)==0)
	    opacity=newMax;
	else
	    opacity = (float) 
		Math.min(1.0, weight/(currentMax-currentMin)*(newMax-newMin)+newMin);
    }

    /*
      public void translate(int dx, int dy){
      x += dx;
      y += dy;
      }
    */

    public int getX(){
	return x;
    }

    public int getY(){
	return y;
    }

    public int getXLast(){
	return xLast;
    }

    public int getYLast(){
	return yLast;
    }

    /**
     * Used to adjust the height of column coloring SparsePixels.
     * @param y an int
     */
    protected void setYLast(int y){
	yLast=y;
    }

    public void printColorParamters(){
	System.out.print("C: ");
	for(int i=0; i<3; i++)
	    System.out.print(rgb[i]+", ");
	System.out.print(opacity);	
	System.out.println(", number of cells: "+getNumberOfIMatrixCells());
    }

    /*
      public Color getColor(){
      //	printColorParamters();

      //	if(rgb[0]>1 || rgb[1]>1 || rgb[2]>1)
      //	    printColorParamters();
      if(rgb==null)
      return null;
      return new Color(rgb[0], rgb[1], rgb[2], opacity);
      }
    */

    public Color getColor(Color defaultColor){
	Color color=null;
	if(rgb==null){
	    //System.out.println("------ Default ------");
	    //Color defaultColor=GroupingColoring.getStandardBackground();
	    float[] r=defaultColor.getRGBColorComponents(null);
	    color = new Color(r[0], r[1], r[2], getOpacity());
	}else{
	    color = new Color(rgb[0], rgb[1], rgb[2], getOpacity());
	}
	return color;
    }

    public float[] getRGB(){
	return rgb;
    }

    public float getOpacity(){
	return opacity;
    }

    public float getWeight(){
	return weight;
    }

    public int getNumberOfIMatrixCells(){
	return matrixCells.size();
    }

    public IMatrixCell getIMatrixCellWithHighestWeight(){
	return matrixCells.first();
    }
	
    public Vector<IMatrixCell> getAllIMatrixCells(){
	return (Vector<IMatrixCell>) new Vector(matrixCells);
    }
	
    public String toString(){
	String ret = "SparsePixel: [x="+x+", y="+y+", ";
	if(rgb!=null)
	    ret += "rgb=["+rgb[0]+", "+rgb[1]+", "+rgb[2]+"], ";
	ret+="o="+opacity+"]";
	return ret;
    }

    private class IMatrixCellWeightComparator implements Comparator{

	/**
	 * Compares two IMatrixCells:s and returns 1, 0 or -1 
	 *
	 * @param  o1 a IMatrixCell
	 * @param  o2 a IMatrixCell
	 * @return int 
	 */
	public int compare(Object o1, Object o2) {
	    IMatrixCell mc1 = (IMatrixCell) o1;
	    IMatrixCell mc2 = (IMatrixCell) o2;

	    // Strst frst
	    if(mc1.getWeight()<mc2.getWeight())
		return 1;
	    else if (mc1.getWeight()>mc2.getWeight())
		return -1;
	    else{
		int cmp = 
		    mc1.getRowIObject().stringCompareTo(mc2.getRowIObject());
		if(cmp!=0)
		    return cmp;
		return mc1.getColumnIObject().stringCompareTo(mc2.getColumnIObject());
	    }
	}    

    }
}

